//
//  NavigationHistory.m
//  MerckManual
//
//  Created by Brian Pfeil on 3/27/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "BookmarkManager.h"

@interface BookmarkManager()
- (void)update;
- (NSString*)cacheFilePath;
- (BOOL)cacheFileExists;
- (BOOL)save;
@end

@implementation BookmarkManager

@synthesize items=_items;

- (void)dealloc {
	[_items release];
	[super dealloc];
}

- (id)init {
	if(self = [super init]) {
		if ([self cacheFileExists]) {
			self = [[NSKeyedUnarchiver unarchiveObjectWithFile:[self cacheFilePath]] retain];
		} else {
			_items = [[NSMutableArray alloc] init];
		}
	}
	return self;
}

+ (BookmarkManager*)sharedInstance {
	static BookmarkManager *sharedInstance = nil;
	if (!sharedInstance) {
		sharedInstance = [[BookmarkManager alloc] init];
	}
	return sharedInstance;
}

- (void)update {
	[self save];
}

- (NSString*)cacheFilePath {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString *filePath = [documentsDirectory stringByAppendingPathComponent:@"bookmarks.archive"];
	return filePath;
}

- (BOOL)cacheFileExists {
	NSString *filePath = [self cacheFilePath];
	return [[NSFileManager defaultManager] fileExistsAtPath:filePath];
}

- (void)addBookmarkNamed:(NSString*)name forItem:(id)item forDate:(NSDate*)date {
	[item setValue:name forKey:@"bookmark_name"];
	[item setValue:date forKey:@"bookmark_date"];
	[_items addObject:item];
	[self save];
}

- (BOOL)save {
	return [NSKeyedArchiver archiveRootObject:self toFile:[self cacheFilePath]];
}

#pragma mark -
#pragma mark NSCoding
- (void)encodeWithCoder:(NSCoder *)encoder {
	[encoder encodeObject:_items forKey:@"items"];
}

- (id)initWithCoder:(NSCoder *)decoder {
	_items = [[decoder decodeObjectForKey:@"items"] retain];
	return self;
}

@end
