//
//  Content.m
//  MerckManual
//
//  Created by Brian Pfeil on 3/27/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "Content.h"
#import "Config.h"

@interface Content(PrivateMethods)
- (void)initDB;
- (NSArray*)getSectionsChaptersTopics;
- (NSArray*)executeQuery:(NSString*)query;
- (NSArray*)executeNamedQuery:(NSString*)queryName;

- (id)getObjectFromArray:(NSArray*)array forKey:(NSString*)key withValue:(NSString*)value;
@end


@implementation Content

@synthesize content=_content;

- (void)dealloc {
	[_db release];
	[_content release];
	[_chaptersAndTopics release];
	[super dealloc];
}

- (id)init {
	if (self = [super init]) {
		[self initDB];
		_content = [[self getContent] retain];
		//_chaptersAndTopics = nil;
	}
	return self;
}

+ (Content*)sharedInstance {
	static Content* sharedInstance = nil;
	if (!sharedInstance) {
		sharedInstance = [[Content alloc] init];
	}
	return sharedInstance;
}

#pragma mark -
#pragma mark database opertations

- (void)initDB {
	NSString *bundlePath = [[NSBundle mainBundle] bundlePath];
	NSString *dbName = [[Config sharedInstance] stringForKeyPath:@"db.name"];
	NSString *dbPath = [bundlePath stringByAppendingPathComponent:dbName];
	_db = [[FMDatabase databaseWithPath:dbPath] retain];
}

- (NSArray*)executeQuery:(NSString*)query {
	
	NSMutableArray *rows = [[NSMutableArray alloc] init];
	
	[_db open];
	FMResultSet *rs = [_db executeQuery:query];
	while ([rs next]) {
        NSMutableDictionary *row = [[NSMutableDictionary alloc] init];
		[rs kvcMagic:row];
		[rows addObject:row];
		[row release];
    }
	[_db close];
	return [rows autorelease];
}

- (NSArray*)executeNamedQuery:(NSString*)queryName {
	NSString *query = [[Config sharedInstance] valueForKeyPath:[NSString stringWithFormat:@"db.sql.%@", queryName]];
	return [self executeQuery:query];
}

#pragma mark -
#pragma mark content operations
- (NSArray*)getContent {
	if (_content) {
		return _content;
	}
	
	@synchronized(self) {
		if ([self contentCacheFileExists]) {
			_content = [[self getContentFromCache] retain];
			return _content;
		}
	}
	
	_content = [[[NSMutableArray alloc] init] retain];
	
	NSArray *rows = [self executeNamedQuery:@"select_sections_chapters_topics"];
	for (NSDictionary *row in rows) {
		NSString *sectionId = [row valueForKey:@"section_id"];
		NSString *sectionName = [row valueForKey:@"section_name"];
		NSString *sectionHref = [row valueForKey:@"section_href"];
		
		NSString *chapterId = [row valueForKey:@"chapter_id"];
		NSString *chapterName = [row valueForKey:@"chapter_name"];
		NSString *chapterHref = [row valueForKey:@"chapter_href"];

		NSString *topicId = [row valueForKey:@"topic_id"];
		NSString *topicName = [row valueForKey:@"topic_name"];
		NSString *topicHref = [row valueForKey:@"topic_href"];
		
		NSMutableDictionary *section = [self getObjectFromArray:_content forKey:@"name" withValue:sectionName];
		if (!section) {
			section = [[NSMutableDictionary alloc] init];
			[section setValue:@"section" forKey:@"type"];
			[section setValue:sectionId forKey:@"id"];
			[section setValue:sectionName forKey:@"name"];
			[section setValue:sectionHref forKey:@"href"];
			[_content addObject:section];
		}
		
		NSMutableArray *chapters = [section valueForKey:@"chapters"];
		if (!chapters) {
			chapters = [[NSMutableArray alloc] init];
			[section setValue:chapters forKey:@"chapters"];
		}
		
		NSMutableDictionary *chapter = [self getObjectFromArray:chapters forKey:@"name" withValue:chapterName];
		if (!chapter) {
			chapter = [[NSMutableDictionary alloc] init];
			[chapter setValue:@"chapter" forKey:@"type"];
			[chapter setValue:sectionId forKey:@"section_id"];
			[chapter setValue:chapterId forKey:@"id"];
			[chapter setValue:chapterName forKey:@"name"];
			[chapter setValue:chapterHref forKey:@"href"];
			[chapters addObject:chapter];
		}
		
		NSMutableArray *topics = [chapter valueForKey:@"topics"];
		if (!topics) {
			topics = [[NSMutableArray alloc] init];
			[chapter setValue:topics forKey:@"topics"];
		}
		
		NSMutableDictionary *topic = (NSMutableDictionary*)[self getObjectFromArray:topics forKey:@"name" withValue:topicName];
		if (!topic) {
			topic = [[NSMutableDictionary alloc] init];
			[topic setValue:@"topic" forKey:@"type"];
			[topic setValue:chapterId forKey:@"chapter_id"];			
			[topic setValue:topicId forKey:@"id"];
			[topic setValue:topicName forKey:@"name"];
			[topic setValue:topicHref forKey:@"href"];
			[topics addObject:topic];
		}
	
	}
	
	[self getAllChaptersAndTopics];
	
	[self save];
	
	return _content;
}

- (NSArray*)getParasForTopicId:(NSString*)topicId {
	NSString *queryTemplate = [[Config sharedInstance] valueForKeyPath:@"db.sql.select_paras_for_topic_by_topic_id"];
	NSString *query = [NSString stringWithFormat:queryTemplate, topicId];
	return [[self executeQuery:query] autorelease];
}

- (NSMutableDictionary*)populateTopic:(NSMutableDictionary*)topic {
	NSString *topicId = [topic valueForKey:@"id"];
	NSArray *rows = [self getParasForTopicId:topicId];
	NSDictionary *row = [rows objectAtIndex:0];
	NSString *chapterId = [row valueForKey:@"chapter_id"];
	NSString *chapterName = [row valueForKey:@"chapter_name"];
	NSString *chapterHref = [row valueForKey:@"chapter_href"];	
	NSString *topicName = [row valueForKey:@"topic_name"];
	NSString *topicHref = [row valueForKey:@"topic_href"];
	
	for (NSMutableDictionary *row in rows) {
		NSString *paraId = [row valueForKey:@"para_id"];
		NSString *paraName = [row valueForKey:@"para_name"];		
		NSString *paraTitle = [row valueForKey:@"para_title"];
		NSString *paraBody = [row valueForKey:@"para_body"];
		[row setValue:@"para" forKey:@"type"];
		[row setValue:topicId forKey:@"topic_id"];		
		[row setValue:paraId forKey:@"id"];
		[row setValue:paraName forKey:@"name"];
		[row setValue:paraTitle forKey:@"title"];
		[row setValue:paraBody forKey:@"body"];
	}
	[topic setValue:chapterId	forKey:@"chapter_id"];
	[topic setValue:chapterName	forKey:@"chapter_name"];
	[topic setValue:chapterHref	forKey:@"chapter_href"];	
	[topic setValue:topicId	forKey:@"id"];
	[topic setValue:topicName forKey:@"name"];
	[topic setValue:topicHref forKey:@"href"];	
	[topic setValue:[rows retain] forKey:@"paras"];
	
	return topic;
}

- (NSArray*)getAllChaptersAndTopics {
	NSMutableArray *items = nil;
	
	@synchronized(self){
	
		if (_chaptersAndTopics) {
			return _chaptersAndTopics;
		}
		
		items = [[NSMutableArray alloc] init];
		NSArray *sections = _content;
		for (NSDictionary *section in sections) {
			NSArray *chapters = [section valueForKey:@"chapters"];
			for (NSDictionary *chapter in chapters) {

				NSString *chapterId = [chapter valueForKey:@"id"];
				NSString *chapterName = [chapter valueForKey:@"name"];
				NSString *chapterHref = [chapter valueForKey:@"href"];
				
				NSArray *topics = [chapter valueForKey:@"topics"];
				for (NSDictionary *topic in topics) {
					
					NSString *topicId = [topic valueForKey:@"id"];
					NSString *topicName = [topic valueForKey:@"name"];
					NSString *topicHref = [topic valueForKey:@"href"];				
					
					NSMutableDictionary *item = [[NSMutableDictionary alloc] init];
					
					[item setValue:chapterId forKey:@"chapter_id"];
					[item setValue:chapterName forKey:@"chapter_name"];
					[item setValue:chapterHref forKey:@"chapter_href"];
					
					[item setValue:topicId forKey:@"topic_id"];
					[item setValue:topicName forKey:@"topic_name"];
					[item setValue:topicHref forKey:@"topic_href"];				
					
					[items addObject:item];
				}
			}
		}
		
		_chaptersAndTopics = [items retain];
	}	
	
	return [items autorelease];
}

- (NSMutableDictionary*)getItemType:(NSString*)type forProperty:(NSString*)property equalToValue:(NSString*)value {
	id result = nil;
	
	if ([type isEqualToString:@"section"]) {
		NSArray *sections = _content;
		for (NSMutableDictionary *section in sections) {
			NSString *propertyValue = [section valueForKey:property];
			if ([propertyValue isEqualToString:value]) {
				result = section;
				break;
			}
		}
	} else if ([type isEqualToString:@"chapter"]) {
		NSArray *sections = _content;
		for (NSMutableDictionary *section in sections) {
			NSArray *chapters = [section valueForKey:@"chapters"];
			for (NSMutableDictionary *chapter in chapters) {
				NSString *propertyValue = [chapter valueForKey:property];
				if ([propertyValue isEqualToString:value]) {
					result = chapter;
					break;
				}				
			}
		}
	} else if ([type isEqualToString:@"topic"]) {
		NSArray *sections = _content;
		for (NSMutableDictionary *section in sections) {
			NSArray *chapters = [section valueForKey:@"chapters"];
			for (NSMutableDictionary *chapter in chapters) {
				NSArray *topics = [chapter valueForKey:@"topics"];
				for (NSMutableDictionary *topic in topics) {
					NSString *propertyValue = [topic valueForKey:property];
					if ([propertyValue isEqualToString:value]) {
						result = topic;
						break;
					}
				}
			}
		}		
	} else if ([type isEqualToString:@"para"]) {
		NSArray *sections = _content;
		for (NSMutableDictionary *section in sections) {
			NSArray *chapters = [section valueForKey:@"chapters"];
			for (NSMutableDictionary *chapter in chapters) {
				NSArray *topics = [chapter valueForKey:@"topics"];
				for (NSMutableDictionary *topic in topics) {
					NSArray *paras = [topic valueForKey:@"paras"];
					for (NSMutableDictionary *para in paras) {
						NSString *propertyValue = [para valueForKey:property];
						if ([propertyValue isEqualToString:value]) {
							result = para;
							break;
						}
					}
				}
			}
		}
		
	}
	
	return result;	
}

- (NSMutableDictionary*)getItemType:(NSString*)type forId:(NSString*)itemId {
	
	return [self getItemType:type forProperty:@"id" equalToValue:itemId];
	
//	id result = nil;
//
//	if ([type isEqualToString:@"section"]) {
//		NSArray *sections = _content;
//		for (NSMutableDictionary *section in sections) {
//			NSString *sectionId = [section valueForKey:@"id"];
//			if ([sectionId isEqualToString:itemId]) {
//				result = section;
//				break;
//			}
//		}
//	} else if ([type isEqualToString:@"chapter"]) {
//		NSArray *sections = _content;
//		for (NSMutableDictionary *section in sections) {
//			NSArray *chapters = [section valueForKey:@"chapters"];
//			for (NSMutableDictionary *chapter in chapters) {
//				NSString *chapterId = [chapter valueForKey:@"id"];
//				if ([chapterId isEqualToString:itemId]) {
//					result = chapter;
//					break;
//				}				
//			}
//		}
//	} else if ([type isEqualToString:@"topic"]) {
//		NSArray *sections = _content;
//		for (NSMutableDictionary *section in sections) {
//			NSArray *chapters = [section valueForKey:@"chapters"];
//			for (NSMutableDictionary *chapter in chapters) {
//				NSArray *topics = [chapter valueForKey:@"topics"];
//				for (NSMutableDictionary *topic in topics) {
//					NSString *topicId = [topic valueForKey:@"id"];
//					if ([topicId isEqualToString:itemId]) {
//						result = topic;
//						break;
//					}
//				}
//			}
//		}		
//	} else if ([type isEqualToString:@"para"]) {
//		NSArray *sections = _content;
//		for (NSMutableDictionary *section in sections) {
//			NSArray *chapters = [section valueForKey:@"chapters"];
//			for (NSMutableDictionary *chapter in chapters) {
//				NSArray *topics = [chapter valueForKey:@"topics"];
//				for (NSMutableDictionary *topic in topics) {
//					NSArray *paras = [topic valueForKey:@"paras"];
//					for (NSMutableDictionary *para in paras) {
//						NSString *paraId = [para valueForKey:@"id"];
//						if ([paraId isEqualToString:itemId]) {
//							result = para;
//							break;
//						}
//					}
//				}
//			}
//		}
//		
//	}
	
//	return result;
}


- (id)getObjectFromArray:(NSArray*)array forKey:(NSString*)key withValue:(NSString*)value {
	id obj = nil;
	
	for (NSMutableDictionary *item in array) {
		NSString *valueForKey = (NSString*)[item valueForKey:key];
		if (valueForKey && [valueForKey isEqualToString:value]) {
			obj = item;
			break;
		}
	}
	
	return obj;		
}

- (BOOL)array:(NSArray*)array hasKey:(NSString*)key withValue:(NSString*)value {
	return ([self getObjectFromArray:array forKey:key withValue:value] != nil);
}

#pragma mark -
#pragma mark cache operations
- (NSString*)contentCacheFilePath {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString *filePath = [documentsDirectory stringByAppendingPathComponent:@"content.plist"];
	return filePath;
}

- (BOOL)contentCacheFileExists {
	NSString *filePath = [self contentCacheFilePath];
	return [[NSFileManager defaultManager] fileExistsAtPath:filePath];
}

- (BOOL)save {
	NSMutableDictionary *container = [[NSMutableDictionary alloc] init];
	[container setValue:_content forKey:@"content"];
	[container setValue:_chaptersAndTopics forKey:@"chaptersAndTopics"];
	BOOL result = [container writeToFile:[self contentCacheFilePath] atomically:YES];
	[container release];
	return result;
}

- (id)getContentFromCache {
	NSString *filePath = [self contentCacheFilePath];
	NSData *data = [[NSData alloc] initWithContentsOfFile:filePath];
	NSString *errorDescription;
	id propertyListData = [NSPropertyListSerialization propertyListFromData:data mutabilityOption:NSPropertyListMutableContainers format:nil errorDescription:&errorDescription];
	_content = [[propertyListData valueForKey:@"content"] retain];
	_chaptersAndTopics = [[propertyListData valueForKey:@"chaptersAndTopics"] retain];
	[data release];
	return _content;
}


@end
