//
//  MerckManualAppDelegate.m
//  MerckManual
//
//  Created by Brian Pfeil on 3/26/09.
//  Copyright __MyCompanyName__ 2009. All rights reserved.
//

#import "MerckManualAppDelegate.h"
#import "MerckManualViewController.h"
#import "Config.h"
#import "Content.h"
#import "PageBuilder.h"
#import "FMDatabase.h"
#import "FMDatabaseAdditions.h"
#import "BookmarkManager.h"


@implementation MerckManualAppDelegate

@synthesize window;
@synthesize navigationController=_navigationController;
@synthesize disclaimerViewController=_disclaimerViewController;
@synthesize merckManualViewController=_merckManualViewController;
@synthesize addBookmarkViewController=_addBookmarkViewController;
@synthesize bookmarksViewController=_bookmarksViewController;
@synthesize tocViewController=_tocViewController;
@synthesize searchViewController=_searchViewController;
@synthesize toolbarActivityIndicator=_toolbarActivityIndicator;
@synthesize activityLabel=_activityLabel;
@synthesize toolbar=_toolbar;

- (void)dealloc {
	[_navigationController release];
	[_disclaimerViewController release];
    [_merckManualViewController release];	
	[_addBookmarkViewController release];
	[_bookmarksViewController release];
	[_tocViewController release];
	[_searchViewController release];
    [window release];
    [super dealloc];
}

- (void)applicationDidFinishLaunching:(UIApplication *)application {
	_navHist = [NavigationHistory sharedInstance];
		
	[self initNavigationController];

	[self initDisclaimerView];
	[self initMerckManualView];
	[self initAddBookmarkView];
	[self initBookmarksView];
	[self initTOCView];
	[self initSearchView];
	
	
    // Override point for customization after app launch    
    [window addSubview:_navigationController.view];
    [window makeKeyAndVisible];
	
	CGRect appFrame = [[UIScreen mainScreen] applicationFrame];
	const CGFloat toolbarHeight = 44;
	
	NSString *systemVersion = [UIDevice currentDevice].systemVersion;
	CGFloat toolbarOffset = 0;
	if ([systemVersion isEqualToString:@"3.0"]) {
		toolbarOffset = 22;
	}
	self.toolbar = [[UIToolbar alloc] initWithFrame:CGRectMake(appFrame.origin.x, appFrame.size.height-(toolbarHeight-toolbarOffset), appFrame.size.width, toolbarHeight)];	
	[self initToolbar];	
	[_navigationController.view addSubview:_toolbar];
	
	[NSTimer scheduledTimerWithTimeInterval:0.5 target:self selector:@selector(loadInitialContent) userInfo:nil repeats:NO];
}

- (void)loadInitialContent {
	// the delegate must be set after the window loads to ensure there is an instance of the webview
	_merckManualViewController.webView.delegate = self;
	
	NSBundle *bundle = [NSBundle mainBundle];
	NSString * urlPathString;
	if (urlPathString = [bundle pathForResource:@"index" ofType:@"html" inDirectory:@"www"]){
		[_merckManualViewController.webView  loadRequest:[NSURLRequest
														  requestWithURL:[NSURL fileURLWithPath:urlPathString]
														  cachePolicy:NSURLRequestUseProtocolCachePolicy
														  timeoutInterval:20.0
														  ]];
	}
	
	[self navDisclaimer:self];
	[self updateNavigation];
	
	id currentItem = [[NavigationHistory sharedInstance] currentItem];
	if (currentItem) {
		[self displayTopicPage:currentItem];
	}
	
	[self showLoadingToolbarWithMessage:@"Loading index ..."];
	[self performSelectorInBackground:@selector(prefetchCachedData:) withObject:nil];
}

- (void)prefetchCachedData:(id)sender {
	[self showLoadingToolbarWithMessage:@"Loading index ..."];
	NSAutoreleasePool *pool = [[NSAutoreleasePool alloc] init];
	[[Content sharedInstance] getContent];
	[NSThread sleepForTimeInterval:5];
	[pool release];
	[self enableNavigation:YES];
	[self showApplicationToolbar];
}

#pragma mark -
#pragma mark View Setup
- (void)initNavigationController {
	self.navigationController = [[UINavigationController alloc] initWithRootViewController:_merckManualViewController];
    _navigationController.navigationBarHidden = YES;
	//_navigationController.toolbarHidden = NO;	
}

- (void)initToolbar {
	
	self.toolbarActivityIndicator = [[UIActivityIndicatorView alloc] initWithFrame:CGRectMake(0.0f, 0.0f, 20.0f, 20.0f)];
	[_toolbarActivityIndicator setActivityIndicatorViewStyle:UIActivityIndicatorViewStyleWhite];
	_activityIndicatorButton = [[UIBarButtonItem alloc] initWithCustomView:_toolbarActivityIndicator];
	
	self.activityLabel = [[UILabel alloc] initWithFrame:CGRectMake(0.0f, 0.0f, 100.0f, 20.0f)];
	_activityLabel.backgroundColor = [UIColor clearColor];
	_activityLabel.font = [UIFont boldSystemFontOfSize:12];
	_activityLabel.textColor = [UIColor whiteColor];
	_activityLabelButton = [[UIBarButtonItem alloc] initWithCustomView:_activityLabel];
	
	// create toolbar items
	_backButton = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"back.png"] style:UIBarButtonItemStylePlain target:self action:@selector(navBack:)];
	_forwardButton = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"forward.png"] style:UIBarButtonItemStylePlain target:self action:@selector(navForward:)];
	_addButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemAdd target:self action:@selector(navAddBookmark:)];
	_bookmarkButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemBookmarks target:self action:@selector(navBookmarks:)];
	_tocButton = [[UIBarButtonItem alloc] initWithImage:[UIImage imageNamed:@"toc.png"] style:UIBarButtonItemStylePlain target:self action:@selector(navTOC:)];
	_searchButton = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemSearch target:self action:@selector(navSearch:)];	
	
	// used to space toolbar buttons evenly
	UIBarButtonItem *flexibleSpace = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
	
	[self showLoadingToolbarWithMessage:@"Loading index ..."];
}

- (void)showLoadingToolbarWithMessage:(NSString*)message {
	_activityLabel.text = message;
	NSArray *toolbarItems = [self getActivityToolbarItems];
	[_toolbar setItems:toolbarItems animated:YES];
	[_toolbarActivityIndicator startAnimating];
	[self enableNavigation:NO];
	_activityIndicatorButton.enabled = YES;
	_activityLabelButton.enabled = YES;
}

- (void)showApplicationToolbar {
	NSArray *toolbarItems = [self getApplicationToolbarItems];
	[_toolbar setItems:toolbarItems animated:YES];
	[_toolbarActivityIndicator stopAnimating];	
	[self updateNavigation];
}

- (NSArray*)getApplicationToolbarItems {
	// used to space toolbar buttons evenly
	UIBarButtonItem *flexibleSpace = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
	
	NSArray *toolbarItems = [NSArray arrayWithObjects:flexibleSpace,
							 _backButton,
							 flexibleSpace,
							 _forwardButton,
							 flexibleSpace,
							 _addButton,
							 flexibleSpace,
							 _bookmarkButton,
							 flexibleSpace,
							 _tocButton,
							 flexibleSpace,
							 _searchButton,
							 flexibleSpace,
							 nil];
	
	[flexibleSpace release];
	return toolbarItems;	
}

- (NSArray*)getActivityToolbarItems {
	// used to space toolbar buttons evenly
	UIBarButtonItem *flexibleSpace = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemFlexibleSpace target:nil action:nil];
	
	NSArray *toolbarItems = [NSArray arrayWithObjects:flexibleSpace,
							 _activityIndicatorButton,
							 _activityLabelButton,
							 flexibleSpace,
							 nil];
	
	[flexibleSpace release];
	return toolbarItems;
}

- (void)initDisclaimerView {
	self.disclaimerViewController = [[DisclaimerViewController alloc] initWithNibName:@"DisclaimerView" bundle:nil];
}

- (void)initMerckManualView {
	
}

- (void)initAddBookmarkView {
	self.addBookmarkViewController = [[AddBookmarkViewController alloc] initWithNibName:@"AddBookmarkView" bundle:nil];
	_addBookmarkViewController.title = @"Add Bookmark";
	_addBookmarkViewController.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Cancel" style:UIBarButtonItemStylePlain target:self action:@selector(cancelView:)];	
}

- (void)initBookmarksView {
	self.bookmarksViewController = [[BookmarksViewController alloc] initWithNibName:@"BookmarksView" bundle:nil];
	_bookmarksViewController.content = [BookmarkManager sharedInstance].items;
	_bookmarksViewController.title = @"Bookmarks";
	_bookmarksViewController.navigationItem.leftBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Cancel" style:UIBarButtonItemStylePlain target:self action:@selector(cancelView:)];	
}

- (void)initTOCView {
	self.tocViewController = [[TOCViewController alloc] initWithNibName:@"TOCView" bundle:nil];
	_tocViewController.title = @"Contents";
	_tocViewController.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Cancel" style:UIBarButtonItemStylePlain target:self action:@selector(cancelView:)];	
}

- (void)initSearchView {
	self.searchViewController = [[SearchViewController alloc] initWithNibName:@"SearchView" bundle:nil];
	_searchViewController.title = @"Search";
	_searchViewController.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Close" style:UIBarButtonItemStylePlain target:self action:@selector(cancelView:)];	
}


#pragma mark -
#pragma mark Navigation
- (void)updateNavigation {
	[self enableNavigation:YES];
	
	_backButton.enabled = [_navHist canNavigateBack];
	_forwardButton.enabled = [_navHist canNavigateForward];
	
	id currentItem = [_navHist currentItem];
	_addButton.enabled = currentItem != nil;
}

- (void)enableNavigation:(BOOL)enabled {
	for (UIBarButtonItem *toolbarItem in _toolbar.items) {
		toolbarItem.enabled = enabled;
	}
}

- (void)startPrefetchData:(id)sender {
	[self enableNavigation:NO];
}

- (void)finishPrefetchData:(id)sender {
	[self enableNavigation:YES];
	[self updateNavigation];
}


- (void)navDisclaimer:(id)sender {
	UINavigationController *nc = [[UINavigationController alloc] initWithRootViewController:_disclaimerViewController];
	[_navigationController presentModalViewController:nc animated:YES];
	[nc release];
}


- (void)nav:(id)item {
	[[Content sharedInstance] populateTopic:item];
	[_navHist navigate:item];
	[self displayTopicPage:item];
	[self updateNavigation];
}

- (void)navBack:(id)sender {
	id item = [_navHist back];	
	[self displayTopicPage:item];
	[self updateNavigation];
}

- (void)navForward:(id)sender {
	id item = [_navHist forward];	
	[self displayTopicPage:item];
	[self updateNavigation];
}

- (void)navAddBookmark:(id)sender {
	UINavigationController *nc = [[[UINavigationController alloc] initWithRootViewController:_addBookmarkViewController] retain];
	_addBookmarkViewController.currentItem = [_navHist currentItem];
	[_navigationController presentModalViewController:nc animated:YES];
	[nc release];	
	
}

- (void)navBookmarks:(id)sender {
	UINavigationController *nc = [[[UINavigationController alloc] initWithRootViewController:_bookmarksViewController] retain];
	[_navigationController presentModalViewController:nc animated:YES];
	[nc release];	
}

- (void)navTOC:(id)sender {
	
	// clear out
	_tocViewController.currentItem = nil;
	
	id item = [_navHist currentItem];
	if (item) {
		_tocViewController.currentItem = item;
		_tocViewController.content = [item valueForKey:@"paras"];
		_tocViewController.sectionHeaderName = [item valueForKey:@"name"];
	} else {
		_tocViewController.content = [Content sharedInstance].content;
		_tocViewController.sectionHeaderName = @"Sections";
	}
	
	UINavigationController *nc = [[[UINavigationController alloc] initWithRootViewController:_tocViewController] retain];
	[_navigationController presentModalViewController:nc animated:YES];
	[nc release];
}

- (void)cancelView:(id)sender {
	[_navigationController dismissModalViewControllerAnimated:YES];
}


- (void)navSearch:(id)sender {
	_searchViewController.content = [[Content sharedInstance] getAllChaptersAndTopics];
	UINavigationController *nc = [[[UINavigationController alloc] initWithRootViewController:_searchViewController] retain];
	[_navigationController presentModalViewController:nc animated:YES];
	[nc release];	
}



#pragma mark -
#pragma mark Page Display
- (void)displayTopicPage:(NSDictionary*)topic {
	[self enableNavigation:NO];
	PageBuilder *pb = [PageBuilder sharedInstance];
	NSString *pageHtml = [pb pageForTopic:topic];
	
	NSBundle *bundle = [NSBundle mainBundle];
	NSString *bundlePath = [bundle bundlePath];
	NSString *wwwDirectoryPath = [bundlePath stringByAppendingPathComponent:@"www"];
	
	[_merckManualViewController.webView loadHTMLString:pageHtml baseURL:[NSURL fileURLWithPath:wwwDirectoryPath]];
	[_merckManualViewController.webView setNeedsDisplay];
	[self enableNavigation:YES];	
}

#pragma mark -
#pragma mark UIWebViewDelegate
- (void)webView:(UIWebView *)webView didFailLoadWithError:(NSError *)error {
}

- (BOOL)webView:(UIWebView *)webView shouldStartLoadWithRequest:(NSURLRequest *)request navigationType:(UIWebViewNavigationType)navigationType {
	// e.g. <NSMutableURLRequest applewebdata://30665D31-3278-4356-AA94-6E7B2AFAA792/mmhe/sec18/ch202/ch202b.html#sec18-ch202-ch202b-22>
	
	NSMutableDictionary *topic = nil;
	if (navigationType == UIWebViewNavigationTypeLinkClicked) {
		NSURL *url = [request mainDocumentURL];
		NSString *path = [url path]; // topic_href
		NSString *fragment = [url fragment]; // para_name
		if (fragment) {
			topic = [[Content sharedInstance] getItemType:@"topic" forProperty:@"href" equalToValue:path];
			[self nav:topic];
		}
		return NO;
	} else {
		return YES;	
	}
}

- (void)webViewDidFinishLoad:(UIWebView *)webView {
}

- (void)webViewDidStartLoad:(UIWebView *)webView {
}

@end
