//
//  NavigationHistory.m
//  MerckManual
//
//  Created by Brian Pfeil on 3/27/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "NavigationHistory.h"

@interface NavigationHistory()
- (void)update;
- (NSString*)cacheFilePath;
- (BOOL)cacheFileExists;
- (BOOL)save;
@end

@implementation NavigationHistory

- (void)dealloc {
	[_items release];
	[super dealloc];
}

- (id)init {
	if(self = [super init]) {
		if ([self cacheFileExists]) {
			self = [[NSKeyedUnarchiver unarchiveObjectWithFile:[self cacheFilePath]] retain];
		} else {
			_items = [[NSMutableArray alloc] init];
			_currentIndex = -1;
		}
	}
	return self;
}

+ (NavigationHistory*)sharedInstance {
	static NavigationHistory *sharedInstance = nil;
	if (!sharedInstance) {
		sharedInstance = [[NavigationHistory alloc] init];
	}
	return sharedInstance;
}

- (id)navigate:(id)item {
	NSString *itemId = [item valueForKey:@"id"];
	
	id lastItem = [_items lastObject];
	if (lastItem) {
		NSString *lastItemId = [lastItem valueForKey:@"id"];
		if ([lastItemId isEqualToString:itemId]) {
			return item;
		}
	}
	
	[_items addObject:item];
	_currentIndex = [_items count] - 1;

	[self update];
	
	return item;
}

- (id)back {
	if ([self canNavigateBack]) {
		_currentIndex--;
	}
	
	[self update];
	
	return [self currentItem];
}

- (id)forward {
	if ([self canNavigateForward]) {
		_currentIndex++;
	}
	
	[self update];	
	
	return [self currentItem];
}

- (BOOL)canNavigateBack {
	return _currentIndex > 0;
}

- (BOOL)canNavigateForward {
	return _currentIndex < ([_items count] - 1);
}

- (id)currentItem {
	if (_currentIndex < 0) {
		return nil;
	} else {
		return [_items objectAtIndex:_currentIndex];
	}
}

- (void)update {
	[self save];
}

- (NSString*)cacheFilePath {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDirectory = [paths objectAtIndex:0];
	NSString *filePath = [documentsDirectory stringByAppendingPathComponent:@"navigation_history.archive"];
	return filePath;
}

- (BOOL)cacheFileExists {
	NSString *filePath = [self cacheFilePath];
	return [[NSFileManager defaultManager] fileExistsAtPath:filePath];
}


- (BOOL)save {
	return [NSKeyedArchiver archiveRootObject:self toFile:[self cacheFilePath]];
}

#pragma mark -
#pragma mark NSCoding
- (void)encodeWithCoder:(NSCoder *)encoder {
	[encoder encodeObject:_items forKey:@"items"];
	[encoder encodeInt:_currentIndex forKey:@"currentIndex"];
}

- (id)initWithCoder:(NSCoder *)decoder {
	_items = [[decoder decodeObjectForKey:@"items"] retain];
	_currentIndex = [decoder decodeIntForKey:@"currentIndex"];
	return self;
}

@end
