//
//  PageBuilder.m
//  MerckManual
//
//  Created by Brian Pfeil on 3/27/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "PageBuilder.h"
#import "Config.h"

@interface PageBuilder(Private)

- (void)loadTemplates;

@end


@implementation PageBuilder

@synthesize templates=_templates;

- (void)dealloc {
	[_templates release];
	[super dealloc];
}

- (id)init {
	if (self = [super init]) {
		_templates = [[[NSMutableDictionary alloc] init] retain];
		[self loadTemplates];
	}
	
	return self;
}

+ (PageBuilder*)sharedInstance {
	PageBuilder *sharedInstance = nil;
	if (!sharedInstance) {
		sharedInstance = [[PageBuilder alloc] init];
	}
	return sharedInstance;
}

- (void)loadTemplates {
	NSBundle *bundle = [NSBundle mainBundle];
	NSString *bundlePath = [bundle bundlePath];
	NSString *templatesDirectoryPath = [bundlePath stringByAppendingPathComponent:@"www/templates"];
	NSFileManager *fm = [NSFileManager defaultManager];
	NSArray *subpaths = [fm subpathsAtPath:templatesDirectoryPath];
	for (NSString *subpath in subpaths) {
		NSString *absoluteFilePath = [templatesDirectoryPath stringByAppendingPathComponent:subpath];
		NSString *fileContents = [NSString stringWithContentsOfFile:absoluteFilePath];
		NSString *templateName = [subpath stringByDeletingPathExtension];
		[_templates setValue:fileContents forKey:templateName];
	}
}

-(NSString*)pageForTopic:(NSDictionary*)topic {
	NSString *chapterName = [topic valueForKey:@"chapter_name"];
	NSArray *topicParas = [topic valueForKey:@"paras"];
	NSMutableString *pageHtml = [[NSMutableString alloc] init];
	NSString *topicName = [topic valueForKey:@"name"];
	NSString *topicNameHtml = [NSString stringWithFormat:@"<b>%@: %@</b><br/><hr/>", chapterName, topicName];
	[pageHtml appendString:topicNameHtml];
	
	for(NSDictionary *para in topicParas) {
		NSString *name = [para valueForKey:@"para_name"];
		NSString *title = [para valueForKey:@"para_title"];		
		NSString *body = [para valueForKey:@"para_body"];
		
		NSString *anchorHtml = [NSString stringWithFormat:@"<a name='#%@' />", name];
		[pageHtml appendString:anchorHtml];
		
		NSString *titleHtml = @"";
		if (title) {
			titleHtml = [NSString stringWithFormat:@"<hr/><b>%@</b><br/>", title];
		}
		[pageHtml appendString:titleHtml];
		
		NSString *paraHtml = [NSString stringWithFormat:@"<p>%@</p>", body];
		[pageHtml appendString:paraHtml];
	}
	
	//NSString *topicPageHtmlTemplate = [[Config sharedInstance] stringForKeyPath:@"html.templates.topic_page"];
	NSString *topicPageHtmlTemplate = [_templates valueForKey:@"merck_manual_home_ed_page_template"];
	NSString *documentHtml = [NSString stringWithFormat:topicPageHtmlTemplate, pageHtml];
	
	[pageHtml release];
	return documentHtml;
}

@end
