//
//  TOCViewController.m
//  MerckManual
//
//  Created by Brian Pfeil on 3/26/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "SearchViewController.h"
#import "DataTableViewCell.h"


@implementation SearchViewController

@synthesize tableView=_tableView;
@synthesize waitingView=_waitingView;
@synthesize content=_content;
@synthesize searchBar=_searchBar;
//@synthesize searchDisplayController=_searchDisplayController;
@synthesize filteredContent=_filteredContent;
@synthesize savedSearchTerm=_savedSearchTerm;
@synthesize savedScopeButtonIndex=_savedScopeButtonIndex;
@synthesize searchWasActive=_searchWasActive;

 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
		_searchThread = nil;
    }
    return self;
}


/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/


// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	
	_waitingView = [[UIView alloc] initWithFrame:CGRectMake(0.0, 44.0, 320.0, 480.0)];
	_waitingView.backgroundColor = [UIColor blackColor];
	_waitingView.opaque = NO;
	_waitingView.alpha = 0.0;
	_waitingView.userInteractionEnabled = YES;
	[self.view addSubview:_waitingView];
	

    // set up the search bar
    //_searchBar.scopeButtonTitles = [NSArray arrayWithObjects:@"All", @"Symptoms", nil];
	_searchBar.autocorrectionType = UITextAutocorrectionTypeNo;
	_searchBar.autocapitalizationType = UITextAutocapitalizationTypeNone;
    
    // create the UISearchDisplayController
//    self.searchDisplayController = [[UISearchDisplayController alloc] initWithSearchBar:_searchBar contentsController:self];
//    _searchDisplayController.searchResultsDataSource = self;
//    _searchDisplayController.searchResultsDelegate = self;
//    _searchDisplayController.delegate = self;
	
	// create our filtered list that will be the data source of our search results table
	self.filteredContent = [[NSMutableArray alloc] initWithCapacity: [_content count]];
	[_filteredContent addObjectsFromArray:_content];
    
    // restore the search settings that were saved in didReceiveMemoryWarning
//    if (_savedSearchTerm) {
//        [_searchDisplayController setActive:_searchWasActive];
//        [_searchBar setSelectedScopeButtonIndex:_savedScopeButtonIndex];
//        [_searchBar setText:_savedSearchTerm];
//        
//        self.savedSearchTerm = nil;
//    }
	
	[self.tableView reloadData];
}

- (void)showWaitingView {
	CGContextRef context = UIGraphicsGetCurrentContext();
	[UIView beginAnimations:nil context:context];
	[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
	[UIView setAnimationDuration:0.3];
	[_waitingView setAlpha:0.5];
	[UIView commitAnimations];
}

- (void)hideWaitingView {
	CGContextRef context = UIGraphicsGetCurrentContext();
	[UIView beginAnimations:nil context:context];
	[UIView setAnimationCurve:UIViewAnimationCurveEaseInOut];
	[UIView setAnimationDuration:0.3];
	[_waitingView setAlpha:0.0];
	[UIView commitAnimations];
}


- (void)viewDidUnload {
    // save the state of the search UI so that we can restore it if our view is re-created
    //self.searchWasActive = [_searchDisplayController isActive];
    self.savedSearchTerm = [_searchBar text];
    //self.savedScopeButtonIndex = [_searchBar selectedScopeButtonIndex];
	
    // destroy the UISearchDisplayController. we'll create a new one if the view is reloaded and point it to the new UISearchBar
    //self.searchDisplayController = nil;
    
    // clear out references to outlet views. we'll get a new UISearchBar if the nib is reloaded
    self.searchBar = nil;
	
	// clear the lists, they are recreated in viewDidLoad
	self.content = nil;
	self.filteredContent = nil;
}


- (void)viewWillAppear:(BOOL)animated {
	[super viewWillAppear:animated];
	[_searchBar becomeFirstResponder];
	[self updateUI];
}

- (void)updateUI {
	[_tableView reloadData];
}


// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    //return (interfaceOrientation == UIInterfaceOrientationPortrait);
	return YES;
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
	[self updateUI];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    NSArray *items = _filteredContent;
	NSArray *chapterTopics = [self getChapterTopics:items];
	int count = [chapterTopics count];
	return (count > 0) ? count : 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    NSArray *items = _filteredContent;
	NSArray *chapterTopics = [self getChapterTopics:items];
	int count = [chapterTopics count];
	if (count > 0) {
		return [[chapterTopics objectAtIndex:section] count];
	} else {
		return 0;
	}
}

- (NSMutableArray*)getChapterTopics:(NSArray*)items {
	
	if (_chapterTopicsCache) {
		return _chapterTopicsCache;
	}
	
	NSMutableArray *chapterTopics = [[NSMutableArray alloc] init];
	NSMutableArray *topics = nil;
	NSString *previousChapterName = nil;
	for (NSDictionary *item in items) {
		NSString *chapterName = [item valueForKey:@"chapter_name"];
		
		if (previousChapterName == nil) {
			topics = [[[NSMutableArray alloc] init] retain];
		}
		else if ([chapterName isEqualToString:previousChapterName] && (previousChapterName != nil) ) {
		
		} else {
			[chapterTopics addObject:topics];
			topics = [[[NSMutableArray alloc] init] retain];
		}
		
		[topics addObject:item];
		
		previousChapterName = chapterName;
	}
	
	_chapterTopicsCache = [chapterTopics retain];
	
	return _chapterTopicsCache;
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    NSArray *items = _filteredContent;
	NSArray *chapterTopics = [self getChapterTopics:items];
	int count = [chapterTopics count];
	if (count > 0) {
		NSArray *topics = [chapterTopics objectAtIndex:section];
		NSDictionary *topic = [topics objectAtIndex:0];
		NSString *chapterName = [topic valueForKey:@"chapter_name"];
		return chapterName;
	} else {
		return @"No results found";
	}
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	static NSString *cellID = @"cellid";
	DataTableViewCell *cell = (DataTableViewCell*)[tableView dequeueReusableCellWithIdentifier:cellID];
	if (!cell) {
		cell = [[DataTableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:cellID];
		cell.font = [UIFont boldSystemFontOfSize:14];
	}
	
    NSArray *items = _filteredContent;
	NSMutableArray *chapterTopics = [self getChapterTopics:items];

	NSDictionary *item = [[chapterTopics objectAtIndex:indexPath.section] objectAtIndex:indexPath.row];
	
	//NSString *chapterName = [item valueForKey:@"chapter_name"];
	NSString *topicName = [item valueForKey:@"topic_name"];
	
//	cell.textLabel.text = chapterName;
//	cell.detailTextLabel.text = topicName;
	
	cell.text = topicName;
	
	return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	
	NSArray *items = _filteredContent;
	NSMutableArray *chapterTopics = [self getChapterTopics:items];
	NSDictionary *item = [[chapterTopics objectAtIndex:indexPath.section] objectAtIndex:indexPath.row];
	
	NSString *topicId = [item valueForKey:@"topic_id"];
	NSMutableDictionary *topic = [[Content sharedInstance] getItemType:@"topic" forId:topicId];
	
	[[UIApplication sharedApplication].delegate performSelector:@selector(nav:) withObject:topic];
	[self.navigationController dismissModalViewControllerAnimated:YES];
	
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
}

#pragma mark -
#pragma mark Content Filtering

- (void)filterBySearchText:(NSString*)searchText {
	
	if (_searchThread) {
		if (![_searchThread isFinished]) {
			NSMutableDictionary* threadDict = [_searchThread threadDictionary];
			[threadDict setValue:[NSNumber numberWithBool:YES] forKey:@"ThreadShouldExitNow"];
		}
	}
	
	if (_searchThread) {
		[_searchThread cancel];
		[_searchThread release];
		_searchThread = nil;
	}
	
	_searchThread = [[[NSThread alloc] initWithTarget:self
											selector:@selector(filterContentForSearchText:)
											  object:searchText] retain];
	
	[_searchThread start];

}

- (void)filterContentForSearchText:(NSString*)searchText
{
	BOOL exitNow = NO;
    NSMutableDictionary* threadDict = [[NSThread currentThread] threadDictionary];
	[threadDict setValue:[NSNumber numberWithBool:NO] forKey:@"finished"];
	[threadDict setValue:[NSNumber numberWithBool:exitNow] forKey:@"ThreadShouldExitNow"];
	
	[_chapterTopicsCache release];
	_chapterTopicsCache = nil;
		
	if (_filteredContent) {
		[_filteredContent removeAllObjects];	// clear the filtered array first
	}
	
	if ([searchText isEqualToString:@""]) {
        [_filteredContent addObjectsFromArray:_content];		
	} else {
	
		for (NSDictionary *item in _content) {
			
			NSString *chapterName = [item valueForKey:@"chapter_name"];
			NSString *topicName = [item valueForKey:@"topic_name"];
			NSString *fullName = topicName;
			//NSString *fullName = [NSString stringWithFormat:@"%@ %@", chapterName, topicName];
				
			NSRange range = [fullName rangeOfString:searchText options:NSCaseInsensitiveSearch range:NSMakeRange(0, [fullName length])];
			
			if (range.location != NSNotFound)  {
				[_filteredContent addObject:item];
			}
			
			exitNow = [[threadDict valueForKey:@"ThreadShouldExitNow"] boolValue];
			if (exitNow) {
				break;
			}
		}
	}
	if (!exitNow) {
		[_tableView reloadData];
	}
	[threadDict setValue:[NSNumber numberWithBool:YES] forKey:@"finished"];
}

#pragma mark -
#pragma mark UISearchBarDelegate
- (void)searchBar:(UISearchBar *)searchBar textDidChange:(NSString *)searchText {
	//[NSThread detachNewThreadSelector:@selector(filterContentForSearchText:scope:) toTarget:self withObject:searchText];
	//[self filterContentForSearchText:searchText];
	[self filterBySearchText:searchText];
}

- (void)searchBarBookmarkButtonClicked:(UISearchBar *)searchBar {
}

- (void)searchBarCancelButtonClicked:(UISearchBar *)searchBar {
	
	if (_searchBar.text.length > 0)
    {
		[_chapterTopicsCache release];
		_chapterTopicsCache = nil;
        [_filteredContent removeAllObjects];
        [_filteredContent addObjectsFromArray:_content];
    }
    
	[self hideWaitingView];
    [_tableView reloadData];
    
    [_searchBar resignFirstResponder];
    _searchBar.text = @"";	
}

- (void)searchBarSearchButtonClicked:(UISearchBar *)searchBar {
	[_searchBar resignFirstResponder];
	[self hideWaitingView];	
}

- (BOOL)searchBarShouldBeginEditing:(UISearchBar *)searchBar {
	[self showWaitingView];
	return YES;
}
//
//- (BOOL)searchBarShouldEndEditing:(UISearchBar *)searchBar {
//}

- (void)searchBarTextDidBeginEditing:(UISearchBar *)searchBar {
	_searchBar.showsCancelButton = YES;
}

- (void)searchBarTextDidEndEditing:(UISearchBar *)searchBar {
	_searchBar.showsCancelButton = NO;
}

#pragma mark -
#pragma mark UISearchDisplayController Delegate Methods

//- (BOOL)searchDisplayController:(UISearchDisplayController *)controller shouldReloadTableForSearchString:(NSString *)searchString {
//	
//    [self filterContentForSearchText:searchString scope:[[_searchBar scopeButtonTitles] objectAtIndex:[_searchBar selectedScopeButtonIndex]]];
//    
//    // return YES to cause the search result table view to be reloaded
//    return YES;
//}
//
//
//- (BOOL)searchDisplayController:(UISearchDisplayController *)controller shouldReloadTableForSearchScope:(NSInteger)searchOption {
//    [self filterContentForSearchText:[_searchBar text] scope:[[_searchBar scopeButtonTitles] objectAtIndex:searchOption]];
//    
//    // return YES to cause the search result table view to be reloaded
//    return YES;
//}


- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)dealloc {
    [super dealloc];
}


@end
