//
//  TOCViewController.m
//  MerckManual
//
//  Created by Brian Pfeil on 3/26/09.
//  Copyright 2009 __MyCompanyName__. All rights reserved.
//

#import "TOCViewController.h"
#import "DataTableViewCell.h"


@implementation TOCViewController

@synthesize tableView=_tableView;
@synthesize sectionHeaderName=_sectionHeaderName;
@synthesize content=_content;
@synthesize currentItem=_currentItem;

 // The designated initializer.  Override if you create the controller programmatically and want to perform customization that is not appropriate for viewDidLoad.
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil {
    if (self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil]) {
		_currentItem = nil;
    }
    return self;
}


/*
// Implement loadView to create a view hierarchy programmatically, without using a nib.
- (void)loadView {
}
*/

/*
// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
}
*/

- (void)viewWillAppear:(BOOL)animated {
	[super viewWillAppear:animated];
	
	if (_currentItem) {
		NSString *chapterId = [_currentItem valueForKey:@"chapter_id"];
		NSString *chapterName = [_currentItem valueForKey:@"chapter_name"];		
		NSMutableDictionary *chapter = [[Content sharedInstance] getItemType:@"chapter" forId:chapterId];
		self.content = [chapter valueForKey:@"topics"];
		self.sectionHeaderName = chapterName;
		self.title = @"Topics";
	}
	[self updateUI];
}

- (void)updateUI {
	[_tableView reloadData];

	if (_currentItem) {
		NSString *currentTopicId = [_currentItem valueForKey:@"id"];
//		NSString *chapterId = [_currentItem valueForKey:@"chapter_id"];
//		NSString *chapterName = [_currentItem valueForKey:@"chapter_name"];		
//		NSMutableDictionary *chapter = [[Content sharedInstance] getItemType:@"chapter" forId:chapterId];
		
		DataTableViewCell *cell = nil;
		NSIndexPath *indexPath;
		int rows = [_tableView numberOfRowsInSection:1];
		for (int i = 0; i < rows; i++) {
			indexPath = [NSIndexPath indexPathForRow:i inSection:1];
			cell = (DataTableViewCell*)[_tableView cellForRowAtIndexPath:indexPath];
			NSMutableDictionary *data = cell.data;
			NSString *topicId = [data valueForKey:@"id"];
			if ([currentTopicId isEqualToString:topicId]) {
				break;
			}
		}
		
		[_tableView selectRowAtIndexPath:indexPath animated:YES scrollPosition:UITableViewScrollPositionMiddle];
	}
	
}


// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    //return (interfaceOrientation == UIInterfaceOrientationPortrait);
	return YES;
}

- (void)didRotateFromInterfaceOrientation:(UIInterfaceOrientation)fromInterfaceOrientation {
	[self updateUI];
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
	return 2;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
	if (section == 0) {
		return 1;
	} else {
		return [_content count];
	}
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
	if (section == 0) {
		return @"";
	} else {
		return _sectionHeaderName;
	}	
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
	static NSString *cellID = @"cellid";
	DataTableViewCell *cell = (DataTableViewCell*)[tableView dequeueReusableCellWithIdentifier:cellID];
	if (!cell) {
		cell = [[DataTableViewCell alloc] initWithFrame:CGRectZero reuseIdentifier:cellID];
		cell.font = [UIFont boldSystemFontOfSize:14];
	}
	
	if (indexPath.section == 0 && indexPath.row == 0) {
		cell.text = @"Home";
		cell.accessoryType = UITableViewCellAccessoryNone;
	} else {
		NSDictionary *data = [_content objectAtIndex:indexPath.row];
		NSString *type = [data valueForKey:@"type"];
		cell.data = data;
		
		if ([type isEqualToString:@"para"]) {
			NSString *title = [data valueForKey:@"title"];
			cell.text = title ? title : @"no title";
		} else {
			cell.text = [data valueForKey:@"name"];
		}
		
		if (! ([type isEqualToString:@"topic"] || [type isEqualToString:@"para"]) ) {
			cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
		} else {
			cell.accessoryType = UITableViewCellAccessoryNone;
		}
		
	}
	
	return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath {
	if (indexPath.section == 0) {
		_content = [Content sharedInstance].content;
		_currentItem = nil;
		self.title = @"Contents";
		//[self.navigationItem.leftBarButtonItem setHidesBackButton:YES animated:YES];
		_sectionHeaderName = @"Sections";
		[_tableView reloadData];
		return;
	} else {
		DataTableViewCell *cell = (DataTableViewCell*)[tableView cellForRowAtIndexPath:indexPath];
		NSDictionary *data = (NSDictionary*)cell.data;
		NSString *type = [data valueForKey:@"type"];
		NSString *name = [data valueForKey:@"name"];
		
		
		if ([type isEqualToString:@"section"] || [type isEqualToString:@"chapter"]) {
			
			TOCViewController *vc = [[TOCViewController alloc] initWithNibName:@"TOCView" bundle:nil];
			vc.navigationItem.rightBarButtonItem = [[UIBarButtonItem alloc] initWithTitle:@"Cancel" style:UIBarButtonItemStylePlain target:[UIApplication sharedApplication].delegate action:@selector(cancelView:)];	

			if ([type isEqualToString:@"section"]) {
				vc.content = [data valueForKey:@"chapters"];
				vc.title = @"Chapters";
				vc.sectionHeaderName = [NSString stringWithFormat:@"%@", name];
			}
			
			if ([type isEqualToString:@"chapter"]) {
				vc.content = [data valueForKey:@"topics"];
				vc.title = @"Topics";			
				vc.sectionHeaderName = [NSString stringWithFormat:@"%@", name];				
			}
			
			[self.navigationController pushViewController:vc animated:YES];
		} else if ([type isEqualToString:@"topic"]){
			[self.navigationController dismissModalViewControllerAnimated:YES];
			[[UIApplication sharedApplication].delegate performSelector:@selector(nav:) withObject:data];
			
		} else {
		}
	}
	[tableView deselectRowAtIndexPath:indexPath animated:YES];
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}


- (void)dealloc {
    [super dealloc];
}


@end
