require 'rubygems'
require 'open-uri'
require 'hpricot'
require 'yaml'
require 'sqlite3'
require 'FileUtils'
#require 'active_record' 
require 'pp'

include FileUtils

#ENV['http_proxy'] = 'http://webproxy.merck.com:8080'

#PROTOCOL = 'http'
#HOST='www.merck.com'
#HOST='localhost:4331'
#INDEX_PATH = '/mmpe/index.html'
#ROOT_URL = "#{PROTOCOL}://#{HOST}#{INDEX_PATH}"

HOST='/Users/brianpfeil/Documents/SiteSucker/www.merck.com'
INDEX_PATH = '/mmhe/index.html'
ROOT_URL = "#{HOST}#{INDEX_PATH}"

ENTITIES = [:section, :chapter, :topic, :para]

def parse_topic(topic)
  t = {}
  t[:name] = topic.inner_html
  t[:href] = topic.attributes['href']
  topic_href = topic.attributes['href']
  topic_url_path = "#{HOST}#{topic_href}"
  doc = Hpricot(open(topic_url_path))
  t[:current_topic] = (doc/'.currentTopic').first.inner_html
  
  paragraphs = []
  
  (doc/'.MMpara').each do |para|
    p = {}
    p[:body] = para.inner_html
    
    # look 2 sibling nodes up to see if there is a header node
    # not every paragraph contains a header node
    node = para.previous_sibling
    if (node.attributes['name'])
        p[:name] = node.attributes['name']
    end
    
    node = para.previous_sibling.previous_sibling
    if (node.attributes['class'] == 'MMfHead')
       p[:title] = node.inner_html
    end
    paragraphs << p
  end
  t[:paragraphs] = paragraphs
  t
end

def parse_section(section)
  s = {}
  s[:name] = section[:name]
  s[:href] = section[:href]
  
  section_name = section[:name]
  section_href = section[:href]
  section_url_path = "#{HOST}#{section_href}"
  doc = Hpricot(open(section_url_path))
  chapters = []
  (doc/'a.MMchapterLnk').each do |chapter|
    c = {}
    c[:name] = chapter.inner_html
    c[:href] = chapter.attributes['href']
    topics = []
    (chapter.parent/'a.MMtopicLnk[@href$="html"]').each do |topic|
      topics << parse_topic(topic)
    end
    c[:topics] = topics
    chapters << c
  end
  s[:chapters] = chapters
  s
end

def flatten_section(section)
  rows = []
  section[:chapters].each do |chapter|
    chapter[:topics].each do |topic|
      topic[:paragraphs].each do |para|
        rows << {
          :section_name => section[:name],
          :section_href => section[:href],
          :chapter_name => chapter[:name],
          :chapter_href =>chapter[:href],
          :topic_name => topic[:name],
          :topic_href => topic[:href],
          :para_name => para[:name],
          :para_title => para[:title],
          :para_body => para[:body]          
        }
      end
    end
  end
  rows
end

def section_generator(&block)
  doc = Hpricot(open(ROOT_URL))
  sections = []
  (doc/'ul.MMnavSectionUL li a[@href*="sec"]').each {|e|  sections << {:name => e.inner_html, :href => e.attributes['href']} }
  
  sections.each do |section|
    puts "Start - parsing section #{section[:name]}"
    parsed_section = parse_section(section)
    puts "End - parsing section #{section[:name]}"
    yield parsed_section
  end  
end

def parse_to_data_flattened_files
  
  section_generator do |section|
    rows = flatten_section(section)
    File.open("data/#{section[:name]}.yaml", 'w') do |out|
       YAML.dump(rows, out)
    end
  end
  
end

def parse_to_data_files
  
  section_generator do |section|
    File.open("raw_parsed_data/#{section[:name]}.yaml", 'w') do |out|
       YAML.dump(section, out)
    end
  end
  
end

def data_for_sections(&block)
  Dir['raw_parsed_data/*.yaml'].each do |path|
    section_data = YAML::load_file(path)
    yield section_data
  end
end

def rows_for_sections(&block)
  Dir['data/*.yaml'].each do |path|
    rows = YAML::load_file(path)
    yield rows
  end  
end

def row_count
  total_rows = 0
  rows_for_sections do |rows|
    total_rows = total_rows + rows.size
  end
  total_rows
end

def create_db_with_denormalized_table_from_rows(db_name, table_name, rows)
  #delete previous db
  try rm db_name rescue
  db = SQLite3::Database.new(db_name)
  column_defs = []
  column_defs << "id INTEGER PRIMARY KEY AUTOINCREMENT"
  rows[0].each do |key, value|
    column_defs << "#{key.to_s} text"
  end
  columns = column_defs.join(', ')
  ddl_create_table_statement = "create table #{table_name} (#{columns});"
  db.execute(ddl_create_table_statement)
  
  # add index for every column except paragrah body
  index_counter = 0
  rows[0].keys.each do |key|
    index_statement = "create index idx#{index_counter} on #{table_name} (#{key.to_s})"
    if key != :para_body
      db.execute(index_statement)
      index_counter = index_counter + 1
      puts index_statement
    end
  end
end

def get_entity_columns_from_row(entity_name, row)
  all_key_names = row.keys.collect{|k| k.to_s}
  entity_key_names = all_key_names.select{|kn| kn =~ Regexp.new("^#{entity_name}")}
  entity_key_names_without_entity_prefix = entity_key_names.collect{|kn| arr = kn.split('_'); arr.slice!(0); arr.join('_') }
  entity_key_names_without_entity_prefix
end

def create_db_with_normalized_tables_from_rows(db_name, rows)
  #delete previous db
  try rm db_name rescue
  db = SQLite3::Database.new(db_name)  
  
  index = 0
  prev_entity_name = nil
  index_counter = 0
  ENTITIES.each do |entity|
    entity_name = entity.to_s
    # all_key_names = rows[0].keys.collect{|k| k.to_s}
    #     entity_key_names = all_key_names.select{|kn| kn =~ Regexp.new("^#{entity_name}")}
    entity_key_names_without_entity_prefix = get_entity_columns_from_row(entity_name, rows[0])
    column_defs = []
    column_defs << "id INTEGER PRIMARY KEY AUTOINCREMENT"
    entity_key_names_without_entity_prefix.each do |kn|
      column_defs << "#{kn} text"
    end
    
    columns = column_defs.join(', ')
    ddl_create_table_statement = "create table #{entity_name} (#{columns});"
    db.execute(ddl_create_table_statement)
    
    
    entity_key_names_without_entity_prefix.each do |kn|
      next if kn == 'body' # don't index paragrah body
      index_statement = "create index idx#{index_counter} on #{entity_name} (#{kn})"
      db.execute(index_statement)
      index_counter = index_counter + 1
    end
    
    
    # create join table
    if prev_entity_name && (index < (ENTITIES.size))
      join_table_name = "#{prev_entity_name}_#{entity_name}"
      ddl_create_join_table_statement = "create table #{join_table_name} (id INTEGER PRIMARY KEY AUTOINCREMENT, #{prev_entity_name}_id INTEGER, #{entity_name}_id INTEGER, FOREIGN KEY (#{prev_entity_name}_id) REFERENCES #{prev_entity_name}(id), FOREIGN KEY (#{entity_name}_id) REFERENCES #{entity_name}(id));"
      db.execute(ddl_create_join_table_statement)
      
      join_table_index_statement = "create index idx#{index_counter} on #{join_table_name} (#{prev_entity_name}_id,#{entity_name}_id)"
      index_counter = index_counter + 1
      db.execute(join_table_index_statement)
    end
    
    index = index + 1
    prev_entity_name = entity_name
  end
  
end

def populate_normalized_db_from_section_data(db_name)
  db = SQLite3::Database.new(db_name)
  section_counter = 0
  data_for_sections do |section|
    #break if section_counter == 4
    data = section
    table_name = 'section'
    db.execute("insert into #{table_name} (name, href) values (:name, :href)", {:name => data[:name], :href => data[:href]})
    #section_id = db.get_first_value("select id from #{table_name} where name = :name and href = :href", {:name => data[:name], :href => data[:href]})
    section_id = db.last_insert_row_id() 
    
    section[:chapters].each do |chapter|
      
      data = chapter
      table_name = 'chapter'
      db.execute("insert into #{table_name} (name, href) values (:name, :href)", {:name => data[:name], :href => data[:href]})
      #chapter_id = db.get_first_value("select id from #{table_name} where name = :name and href = :href", {:name => data[:name], :href => data[:href]})
      chapter_id = db.last_insert_row_id() 
      db.execute("insert into section_chapter (section_id, chapter_id) values (:section_id, :chapter_id)", {:section_id => section_id, :chapter_id => chapter_id})
      
       chapter[:topics].each do |topic|
         
         data = topic
         table_name = 'topic'
         db.execute("insert into #{table_name} (name, href) values (:name, :href)", {:name => data[:name], :href => data[:href]})
         #topic_id = db.get_first_value("select id from #{table_name} where name = :name and href = :href", {:name => data[:name], :href => data[:href]})
         topic_id = db.last_insert_row_id() 
         db.execute("insert into chapter_topic (chapter_id, topic_id) values (:chapter_id, :topic_id)", {:chapter_id => chapter_id, :topic_id => topic_id})
         
         topic[:paragraphs].each do |para|

           data = para
           table_name = 'para'
           db.execute("insert into #{table_name} (name, title, body) values (:name, :title, :body)", {:name => data[:name], :title => data[:title], :body => data[:body]})
           #para_id = db.get_first_value("select id from #{table_name} where name = :name and title = :title and body = :body", {:name => data[:name], :title => data[:title], :body => data[:body]})
           para_id = db.last_insert_row_id() 
           puts "para_id = #{para_id}"
           db.execute("insert into topic_para (topic_id, para_id) values (:topic_id, :para_id)", {:topic_id => topic_id, :para_id => para_id})

         end
       end
     end    
    section_counter = section_counter + 1
  end
  
end


def populate_normalized_db_from_flattened_rows(db_name, rows)
  db = SQLite3::Database.new(db_name)

  rows.each do |row|
    prev_entity_name = nil
    entity_index = 0
    ENTITIES.each do |entity|
      entity_name = entity.to_s
      entity_columns = get_entity_columns_from_row(entity_name, row)
      
      sorted_column_names = entity_columns.sort
      columns = sorted_column_names.join(',')
      values = sorted_column_names.collect{|c| ":#{c}"}.join(', ')
      insert_statement = "insert into #{entity_name} (#{columns}) values (#{values})"
      puts insert_statement
      
      if prev_entity_name && (entity_index < ENTITIES.size)
      end
      
      prev_entity_name = entity_name
      entity_index = entity_index + 1
      #db.execute(insert_statement, row)      
    end
    
  end
  
  
  rows = []
  section[:chapters].each do |chapter|
    chapter[:topics].each do |topic|
      topic[:paragraphs].each do |para|
        rows << {
          :section_name => section[:name],
          :section_href => section[:href],
          :chapter_name => chapter[:name],
          :chapter_href =>chapter[:href],
          :topic_name => topic[:name],
          :topic_href => topic[:href],
          :para_name => para[:name],
          :para_title => para[:title],
          :para_body => para[:body]          
        }
      end
    end
  end
  rows  
end

def populate_db_from_rows(db_name, table_name, rows)
  db = SQLite3::Database.new(db_name)
  rows.each do |row|
    sorted_column_names = row.keys.collect{|k| k.to_s}.sort
    columns = sorted_column_names.join(',')
    raw_values = sorted_column_names.collect{|c| row[c.to_sym]}
    values = sorted_column_names.collect{|c| ":#{c}"}.join(', ')
    insert_statement = "insert into #{table_name} (#{columns}) values (#{values})"
    db.execute(insert_statement, row)
  end
end

rows_for_sections do |rows|
    create_db_with_normalized_tables_from_rows("merck_manual.sqlite3", rows)
    break
end

populate_normalized_db_from_section_data("merck_manual.sqlite3")


